@@ -1,20 +1,23 @@
 #
-# $Id: Context.pm 354 2014-11-16 14:25:05Z gomor $
+# $Id: Context.pm,v a5b24eee501b 2016/01/02 17:04:51 gomor $
 #
 package Metabrik::Core::Context;
 use strict;
 use warnings;
 
+our $VERSION = '1.20.0';
+
 use base qw(Metabrik);
 
 sub brik_properties {
    return {
-      revision => '$Revision: 354 $',
-      tags => [ qw(core context main) ],
+      revision => '$Revision: a5b24eee501b $',
+      tags => [ qw(main core) ],
       attributes => {
          _lp => [ qw(INTERNAL) ],
       },
       commands => {
+         new_brik_run => [ qw(Brik Command Args) ],
          use => [ qw(Brik) ],
          set => [ qw(Brik Attribute Value) ],
          get => [ qw(Brik Attribute) ],
@@ -27,6 +30,7 @@ sub brik_properties {
          available => [ ],
          is_available => [ qw(Brik) ],
          used => [ ],
+         get_used => [ qw(Brik) ],
          is_used => [ qw(Brik) ],
          not_used => [ ],
          is_not_used => [ qw(Brik) ],
@@ -36,8 +40,9 @@ sub brik_properties {
          restore_state => [ qw(Brik) ],
       },
       require_modules => {
-         'Lexical::Persistence' => [ ],
+         'Data::Dump' => [ qw(dump) ],
          'File::Find' => [ ],
+         'Lexical::Persistence' => [ ],
          'Module::Reload' => [ ],
          'Metabrik::Core::Global' => [ ],
          'Metabrik::Core::Log' => [ ],
@@ -148,18 +153,24 @@ sub new {
    return $self->brik_preinit;
 }
 
+sub new_brik_run {
+   my $self = shift;
+   my ($brik, $command, @args) = @_;
+
+   my $con = Metabrik::Core::Context->new or return;
+
+   $con->use($brik) or return;
+   my $data = $con->run($brik, $command, @args) or return;
+   $con->brik_fini;
+
+   # Compatibility with file::dump Brik
+   print Data::Dump::dump($data)."\n";
+
+   return $con;
+}
+
 sub brik_init {
-   my $self = shift->SUPER::brik_init(
-      @_,
-   ) or return 1; # Init already done
-
-   my $on_int = sub {
-       $self->debug && $self->log->debug("INT captured");
-       return 1;
-   };
-
-   $SIG{INT} = $on_int;
-   $SIG{TERM} = $on_int;
+   my $self = shift;
 
    my $r = $self->update_available;
    if (! defined($r)) {
@@ -168,7 +179,7 @@ sub brik_init {
       );
    }
 
-   return $self;
+   return $self->SUPER::brik_init(@_);
 }
 
 sub do {
@@ -214,7 +225,9 @@ sub call {
       my @list = caller();
       my $file = $list[1];
       my $line = $list[2];
-      #return $self->log->error("call: $@ (source file [$file] at line [$line])");
+      if ($self->debug) {
+         return $self->log->error("call: $@ (source file [$file] at line [$line])");
+      }
       return $self->log->error("call: $@");
    }
 
@@ -415,9 +428,48 @@ sub use {
 sub reuse {
    my $self = shift;
 
-   my $reused = Module::Reload->check;
-   if ($reused) {
-      $self->log->info("reuse: some modules were reused");
+   my %stat = ();
+   my @reloaded = ();
+   # Taken from Module::Reload
+   for my $entry (map { [ $_, $INC{$_} ] } keys %INC) {
+      my ($module, $file) = @$entry;
+
+      if ($file eq $INC{"Module/Reload.pm"}) {
+         next;   # Too confusing
+      }
+
+      local $^W = 0;  # Disable 'use warnings';
+      my $mtime = (stat $file)[9];
+      if (! defined($stat{$file})) {
+         $stat{$file} = $^T;
+      }
+
+      next unless defined($mtime);
+
+      if ($mtime > $stat{$file}) {
+         delete $INC{$module};
+         eval { 
+            $SIG{__WARN__} = sub {};
+            require $module;
+         };
+         if ($@) {
+            chomp($@);
+            if ($self->debug) {
+               $self->log->error("reuse: reloading module [$module] failed: [$@]");
+            }
+            else {
+               $self->log->error("reuse: reloading module [$module] failed");
+            }
+         }
+         else {
+            push @reloaded, $module;
+         }
+      }
+      $stat{$file} = $mtime;
+   }
+
+   for (@reloaded) {
+      $self->log->verbose("reuse: module [$_] successfully reloaded");
    }
 
    return 1;
@@ -459,6 +511,24 @@ sub used {
    return $r;
 }
 
+sub get_used {
+   my $self = shift;
+   my ($brik) = @_;
+
+   if (! defined($brik)) {
+      return $self->log->error($self->brik_help_run('get_used'));
+   }
+
+   my $used = $self->used;
+
+   my $get = $used->{$brik};
+   if (! defined($get)) {
+      return $self->log->error("get_used: Brik [$brik] not used");
+   }
+
+   return $get;
+}
+
 sub is_used {
    my $self = shift;
    my ($brik) = @_;
@@ -574,8 +644,33 @@ sub set {
          die("$MSG\n");
       }
 
-      if ($__ctx_value =~ /^(\$.*)$/) {
-         $__ctx_value = eval("\$CON->_lp->{context}->{_}->{'$1'}");
+      # Support variable lookups like '$array' as an Argument
+      # Example: set <Brik> <Attribute> $Arg
+      if ($__ctx_value =~ /^\$\w+/ || $__ctx_value =~ /^\@\$\w+/
+      ||  $__ctx_value =~ /^\@\w+/ || $__ctx_value =~ /^\%\$\w+/
+      ||  $__ctx_value =~ /^\%\w+/) {
+         eval {
+            $__ctx_value = $CON->_lp->do($__ctx_value);
+         };
+         if ($@) {
+            chomp($@);
+            $ERR = 1;
+            my $MSG = "set: Brik [$__ctx_brik] has invalid argument [$__ctx_value]";
+            die("$MSG\n");
+         }
+      }
+      # Support passing ARRAYs or HASHs or Perl code as an Argument
+      # Example: set <Brik> <Attribute> "[ qw(a b c) ]"
+      elsif ($__ctx_value =~ /^\[.*\]$/ || $__ctx_value =~ /^\{.*\}$/) {
+         eval {
+            $__ctx_value = $CON->_lp->do($__ctx_value);
+         };
+         if ($@) {
+            chomp($@);
+            $ERR = 1;
+            my $MSG = "set: Brik [$__ctx_brik] has invalid argument [$__ctx_value]";
+            die("$MSG\n");
+         }
       }
 
       $CON->{used}->{$__ctx_brik}->$__ctx_attribute($__ctx_value);
@@ -663,19 +758,54 @@ sub run {
 
       my $__ctx_run = $CON->{used}->{$__ctx_brik};
 
-      $__ctx_run->brik_init; # Will brik_init() only if not already done
-
-      for (@__ctx_args) {
-         if (/^(\$.*)$/) {
-            $_ = eval("\$CON->_lp->{context}->{_}->{'$1'}");
+      # Will brik_init() only if not already done
+      # And only for Brik's Commands, not base class Commands
+      if (! $__ctx_run->init_done && $__ctx_command !~ /^brik_/) {
+         if (! $__ctx_run->brik_init) {
+            $ERR = 1;
+            my $MSG = "run: Brik [$__ctx_brik] init failed";
+            die("$MSG\n");
          }
       }
 
-      my $RUN = $__ctx_run->$__ctx_command(@__ctx_args);
-      if (! defined($RUN)) {
-         $ERR = 1;
+      for (@__ctx_args) {
+         # Support variable lookups like '$array' as an Argument
+         # Example: run <Brik> <Command> $Arg1 Arg2
+         if (/^\$\w+/ || /^\@\$\w+/ || /^\@\w+/ || /^\%\$\w+/ || /^\%\w+/) {
+            eval {
+               $_ = $CON->_lp->do($_);
+            };
+            if ($@) {
+               chomp($@);
+               $ERR = 1;
+               my $MSG = "run: Brik [$__ctx_brik] has invalid argument [$_]";
+               die("$MSG\n");
+            }
+         }
+         # Support passing ARRAYs or HASHs or Perl code as an Argument
+         # Example: run <Brik> <Command> "[ qw(a b c) ]"
+         elsif (/^\[.*\]$/ || /^\{.*\}$/) {
+            eval {
+               $_ = $CON->_lp->do($_);
+            };
+            if ($@) {
+               chomp($@);
+               $ERR = 1;
+               my $MSG = "run: Brik [$__ctx_brik] has invalid argument [$_]";
+               die("$MSG\n");
+            }
+         }
       }
 
+      my $RUN;
+      my $__ctx_return = $__ctx_run->$__ctx_command(@__ctx_args);
+      if (! defined($__ctx_return)) {
+         $ERR = 1;
+         return;
+      }
+
+      $RUN = $__ctx_return;
+
       my $REF = \$RUN;
 
       return $RUN;
@@ -706,13 +836,13 @@ sub save_state {
       }
 
       my $__ctx_state;
-      my $__ctx_attributes = $CON->{used}->{$__ctx_brik}->brik_properties->{attributes} || {};
+      my $__ctx_attributes = $CON->{used}->{$__ctx_brik}->brik_attributes || {};
       for my $__ctx_this (keys %$__ctx_attributes) {
          $__ctx_state->{$__ctx_this} = $CON->{used}->{$__ctx_brik}->$__ctx_this;
       }
       $CON->{used}->{$__ctx_brik}->{"__ctx_state"} = $__ctx_state;
 
-      return $ERR;
+      return 1;
    }, brik => $brik);
 
    return $r;
@@ -746,12 +876,24 @@ sub restore_state {
          }
       }
 
-      return $ERR;
+      return 1;
    }, brik => $brik);
 
    return $r;
 }
 
+sub brik_fini {
+   my $self = shift;
+
+   my $used = $self->used;
+   for my $brik (keys %$used) {
+      next if $brik eq 'core::context'; # Avoid recursive loop
+      $used->{$brik}->brik_fini;
+   }
+
+   return 1;
+}
+
 1;
 
 __END__
@@ -760,9 +902,95 @@ __END__
 
 Metabrik::Core::Context - core::context Brik
 
+=head1 SYNOPSIS
+
+   use Metabrik::Core::Context;   
+
+   my $CON = Metabrik::Core::Context->new or die("core::context");
+
+=head1 DESCRIPTION
+
+This Brik keeps track of everything that happens within B<Metabrik>. That includes persistence of Perl variables and loaded Briks along with their set Attributes for instance.
+
+This is the only Brik that is mandatory to B<use> when writing a Tool, and it will auto-load B<core::log>, B<core::global> and B<core::shell> for you. When these Briks are loaded from B<core::context>, some global variables are set as pointers to them: B<$CON>, B<$GLO>, B<$LOG>, and B<$SHE> to point to respectively B<core::context>, B<core::global>, B<core::log> and B<core::shell> Briks.
+
+=head1 ATTRIBUTES
+
+At B<The Metabrik Shell>, just type:
+
+L<get core::context>
+
+=head1 COMMANDS
+
+At B<The Metabrik Shell>, just type:
+
+L<help core::context>
+
+=head1 METHODS
+
+=over 4
+
+=item B<brik_properties>
+
+=item B<new>
+
+=item B<new_brik_run>
+
+=item B<brik_init>
+
+=item B<available>
+
+=item B<call>
+
+=item B<do>
+
+=item B<find_available>
+
+=item B<get>
+
+=item B<get_used>
+
+=item B<is_available>
+
+=item B<is_not_used>
+
+=item B<is_used>
+
+=item B<log>
+
+=item B<not_used>
+
+=item B<restore_state>
+
+=item B<reuse>
+
+=item B<run>
+
+=item B<save_state>
+
+=item B<set>
+
+=item B<status>
+
+=item B<update_available>
+
+=item B<use>
+
+=item B<used>
+
+=item B<variables>
+
+=item B<brik_fini>
+
+=back
+
+=head1 SEE ALSO
+
+L<Metabrik>
+
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2014-2016, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of The BSD 3-Clause License.
 See LICENSE file in the source distribution archive.
